/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "RawDC.h"

#include <libopenraw/libopenraw.h>

#include <GTLCore/Macros_p.h>
#include <GTLImageIO/ImageDCRegistry.h>
#include <GTLCore/Image.h>
#include <GTLCore/Buffer.h>
#include <GTLCore/Type.h>
#include <GTLCore/Region.h>
#include <GTLCore/Debug.h>
#include <GTLCore/PixelDescription.h>

class OpenRawBuffer : public GTLCore::Buffer {
  public:
    OpenRawBuffer(ORRawDataRef rawData) : m_rawData( rawData )
    {
    }
    virtual ~OpenRawBuffer()
    {
      or_rawdata_release( m_rawData );
    }
    virtual char * rawData()
    {
      return reinterpret_cast<char*>( or_rawdata_data( m_rawData ) );
    }
    virtual const char * rawData() const
    {
      return reinterpret_cast<const char*>( or_rawdata_data( m_rawData ) );
    }
    
    virtual int size() const
    {
      return or_rawdata_data_size( m_rawData );
    }
    
      
  private:
    ORRawDataRef m_rawData;
};


STATIC_INITIALISATION( RawDC )
{
  GTLImageIO::ImageDCRegistry::instance()->registerDC( new RawDC );
}

RawDC::RawDC()
{
  addReadWriteExtension("nef");
}

RawDC::~RawDC()
{
}

GTLCore::AbstractImage* RawDC::decode( const GTLCore::String& _fileName, GTLCore::Region* _region, GTLCore::String* _errorMessage ) const
{
  ORRawFileRef file = or_rawfile_new( _fileName.c_str(), OR_RAWFILE_TYPE_UNKNOWN );
  ORRawDataRef rawData = or_rawdata_new();
  if( or_rawfile_get_rawdata( file, rawData, OR_OPTIONS_NONE) != OR_ERROR_NONE )
  {
    or_rawfile_release( file );
    return 0;
  }
  
  or_rawfile_release( file );
  uint32_t width, height;
  or_rawdata_dimensions(rawData, &width, &height );
  if(_region)
  {
    _region->setWidth( width );
    _region->setHeight( height );
  }
  return new GTLCore::BufferImage( width, height, new OpenRawBuffer( rawData), GTLCore::PixelDescription( GTLCore::Type::UnsignedInteger16, 1 ) ) ;
}

bool RawDC::encode( const GTLCore::AbstractImage* _image, const GTLCore::Region& _region, const GTLCore::String& _fileName, GTLCore::String* _errorMessage ) const
{
  return false;
}

bool RawDC::canEncodeImage( const GTLCore::String& _fileName ) const
{
  return false;
}
