/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _GTLGlue_PIPELINE_H_
#define _GTLGlue_PIPELINE_H_

#include <GTLGlue/ProcessingNode.h>

namespace GTLCore {
  class PixelDescription;
}

namespace GTLGlue {
  class OutputNode;
  class InputNode;
  /**
   * @ingroup GTLGlue
   * This is the main class for GTLGlue, this class contains the image processing
   * pipeline.
   */
  class Pipeline : public ProcessingNode {
    friend class ProcessingNode;
    public:
      Pipeline();
      ~Pipeline();
    public:
      /**
       * Append a source to the \ref Pipeline.
       */
      void appendSource( const GTLCore::PixelDescription& );
      /**
       * Append a destination to the \ref Pipeline.
       */
      void appendDestination( const GTLCore::PixelDescription& );
      void appendProcessingNode( ProcessingNode* _processingNode );
      OutputNode* sourceNode( unsigned int _index);
      InputNode* destinationNode( unsigned int _index);
    private:
      class Private;
      Private* const d;
  };
}

#endif
