/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "Pipeline_p.h"

#include "GTLCore/Utils_p.h"

#include "Debug_p.h"
#include "DestinationNode_p.h"
#include "SourceNode_p.h"

using namespace GTLGlue;

Pipeline::Private::~Private()
{
  GTLCore::deleteAll( m_sources );
  GTLCore::deleteAll( m_destinations );
  GTLCore::deleteAll( m_nodes );
}

void Pipeline::Private::appendSource( const GTLCore::PixelDescription& _pd)
{
  m_sources.push_back( new SourceNode( _pd ) );
}

void Pipeline::Private::appendDestination( const GTLCore::PixelDescription& _pd )
{
  m_destinations.push_back( new DestinationNode( _pd ) );
}

OutputNode* Pipeline::Private::sourceNode( unsigned int _index)
{
  GTLF_ASSERT( _index < m_sources.size() );
  return m_sources[ _index ];
}

InputNode* Pipeline::Private::destinationNode( unsigned int _index)
{
  GTLF_ASSERT( _index < m_destinations.size() );
  return m_destinations[ _index ];
}

void Pipeline::Private::appendProcessingNode( ProcessingNode* _processingNode )
{
  m_nodes.push_back( _processingNode );
}
